@@ -32,12 +32,18 @@
 #include "dlg.h"
 #include "b2b_entities.h"
 #include "b2be_load.h"
+#include "ua_api.h"
 
+str* _client_new(client_info_t* ci,b2b_notify_t b2b_cback,
+		b2b_add_dlginfo_t add_dlginfo, str *mod_name, str* logic_key,
+		struct ua_sess_init_params *init_params, struct b2b_tracer *tracer,
+		void *param, b2b_param_free_cb free_param);
 str* client_new(client_info_t* ci, b2b_notify_t b2b_cback,
-		b2b_add_dlginfo_t add_dlginfo, str *mod_name, str* param);
+		b2b_add_dlginfo_t add_dlginfo, str *mod_name, str* param,
+		struct b2b_tracer*, void *, b2b_param_free_cb);
 
 void b2b_client_tm_cback( struct cell *t, int type, struct tmcb_params *ps);
 
-dlg_t* b2b_client_build_dlg(b2b_dlg_t* dlg, dlg_leg_t* leg);
+dlg_t* b2b_client_build_dlg(b2b_dlg_t* dlg, dlg_leg_t* leg, unsigned int maxfwd);
 
 #endif
