@@ -31,6 +31,7 @@
 #include "../../mem/mem.h"
 #include "../../db/db.h"
 #include "../../mod_fix.h"
+#include "../../ipc.h"
 #include "../../lib/list.h"
 #include "../../resolve.h"
 #include "cgrates.h"
@@ -73,16 +74,15 @@ static int w_pv_get_cgr(struct sip_msg *msg, pv_param_t *param,
 		pv_value_t *val);
 static int w_pv_get_cgr_opt(struct sip_msg *msg, pv_param_t *param,
 		pv_value_t *val);
-static int pv_parse_cgr(pv_spec_p sp, str *in);
-static int w_pv_parse_cgr(pv_spec_p sp, str *in);
-static int w_pv_parse_cgr_warn(pv_spec_p sp, str *in);
-static int pv_parse_idx_cgr(pv_spec_p sp, str *in);
+static int pv_parse_cgr(pv_spec_p sp, const str *in);
+static int w_pv_parse_cgr(pv_spec_p sp, const str *in);
+static int pv_parse_idx_cgr(pv_spec_p sp, const str *in);
 static int pv_get_cgr_reply(struct sip_msg *msg, pv_param_t *param,
 		pv_value_t *val);
 
 OSIPS_LIST_HEAD(cgrates_engines);
 
-static cmd_export_t cmds[] = {
+static const cmd_export_t cmds[] = {
 	{"cgrates_acc", (cmd_function)w_cgr_acc, {
 		{CMD_PARAM_STR|CMD_PARAM_OPT, fixup_flags, 0},
 		{CMD_PARAM_STR|CMD_PARAM_OPT, fixup_dlg_loaded, 0},
@@ -101,19 +101,17 @@ static cmd_export_t cmds[] = {
 	{0,0,{{0,0,0}},0}
 };
 
-static pv_export_t pvars[] = {
-	{ str_init("cgr"), 2003, w_pv_get_cgr, w_pv_set_cgr,
+static const pv_export_t pvars[] = {
+	{ str_const_init("cgr"), 2003, w_pv_get_cgr, w_pv_set_cgr,
 		pv_parse_cgr, pv_parse_idx_cgr, 0, 0},
-	{ str_init("cgr_opt"), 2004, w_pv_get_cgr_opt, w_pv_set_cgr_opt,
+	{ str_const_init("cgr_opt"), 2004, w_pv_get_cgr_opt, w_pv_set_cgr_opt,
 		w_pv_parse_cgr, pv_parse_idx_cgr, 0, 0},
-	{ str_init("cgr_ret"), 2005, pv_get_cgr_reply, 0,
+	{ str_const_init("cgr_ret"), 2005, pv_get_cgr_reply, 0,
 		pv_parse_cgr, 0, 0, 0},
-	{ str_init("cgrret"), 2005, pv_get_cgr_reply, 0,
-		w_pv_parse_cgr_warn, 0, 0, 0},
 	{ {0, 0}, 0, 0, 0, 0, 0, 0, 0 }
 };
 
-static acmd_export_t acmds[] = {
+static const acmd_export_t acmds[] = {
 	{"cgrates_auth", (acmd_function)w_acgr_auth, {
 		{CMD_PARAM_STR|CMD_PARAM_OPT, 0, 0},
 		{CMD_PARAM_STR|CMD_PARAM_OPT, 0, 0},
@@ -124,7 +122,7 @@ static acmd_export_t acmds[] = {
 	{0,0,{{0,0,0}}}
 };
 
-static param_export_t params[] = {
+static const param_export_t params[] = {
 	{"cgrates_engine", STR_PARAM|USE_FUNC_PARAM,
 		(void*)cgrates_set_engine },
 	{"bind_ip", STR_PARAM, &cgre_bind_ip.s },
@@ -134,7 +132,7 @@ static param_export_t params[] = {
 	{0, 0, 0}
 };
 
-static dep_export_t deps = {
+static const dep_export_t deps = {
 	{ /* OpenSIPS module dependencies */
 		{ MOD_TYPE_DEFAULT, "dialog", DEP_ABORT },
 		{ MOD_TYPE_NULL, NULL, 0 },
@@ -277,10 +275,11 @@ static int child_init(int rank)
 	/* go through each server and initialize a default connection */
 	list_for_each(l, &cgrates_engines) {
 		e = list_entry(l, struct cgr_engine, list);
-		if ((c = cgrc_new(e)) && cgrc_conn(c) >= 0) {
+		if ((c = cgrc_new(e)) != NULL) {
 			e->default_con = c;
 			CGRC_SET_DEFAULT(c);
-			cgrc_start_listen(c);
+			if (ipc_send_rpc(process_no, cgrc_conn_rpc, c) < 0)
+				LM_ERR("could not send connect job!\n");
 		}
 	}
 	return cgr_init_common();
@@ -325,14 +324,10 @@ static int cgrates_set_engine(modparam_t type, void * val)
 		port = CGR_DEFAULT_PORT;
 	}
 	str_trim_spaces_lr(host);
-	if ((ip = str2ip(&host)) == NULL) {
-		LM_ERR("invalid ip in cgr engine host: %.*s\n", host.len, host.s);
-		return -1;
-	}
 
 	LM_DBG("Adding cgrates engine %.*s:%u\n", host.len, host.s, port);
 
-	e = pkg_malloc(sizeof(*e) + host.len);
+	e = pkg_malloc(sizeof(*e) + host.len + 1);
 	if (!e) {
 		LM_ERR("out of pkg mem!\n");
 		return -1;
@@ -341,9 +336,14 @@ static int cgrates_set_engine(modparam_t type, void * val)
 	e->host.s = (char *)(e + 1);
 	e->host.len = host.len;
 	memcpy(e->host.s, host.s, host.len);
+	e->host.s[e->host.len] = '\0';
 
 	e->port = port;
-	init_su(&e->su, ip, port);
+
+	if ((ip = str2ip(&host)) || (ip = str2ip6(&host)))
+		init_su(&e->su, ip, port);
+	else
+		e->is_fqdn = 1;
 
 	INIT_LIST_HEAD(&e->conns);
 
@@ -443,6 +443,7 @@ static int pv_set_cgr(struct sip_msg *msg, pv_param_t *param,
 	} else
 		return 0; /* initialised with NULL */
 
+	kv->flags &= ~(CGR_KVF_TYPE_NULL|CGR_KVF_TYPE_JSON);
 	if (val->flags & PV_VAL_NULL) {
 		kv->flags |= CGR_KVF_TYPE_NULL;
 	} else if (val->flags & PV_VAL_INT) {
@@ -458,6 +459,8 @@ static int pv_set_cgr(struct sip_msg *msg, pv_param_t *param,
 		kv->value.s.len = val->rs.len;
 		kv->flags |= CGR_KVF_TYPE_STR;
 	}
+	if (op == COLONEQ_T)
+		kv->flags |= CGR_KVF_TYPE_JSON;
 	LM_DBG("add cgr kv: %d %s in %p\n", kv->key.len, kv->key.s, s);
 
 	return 0;
@@ -492,7 +495,7 @@ static int pv_get_cgr(struct sip_msg *msg, pv_param_t *param,
 		return -1;
 	}
 
-	if (!(ctx = CGR_GET_CTX()))
+	if (!(ctx = cgr_try_get_ctx()))
 		return pv_get_null(msg, param, val);
 
 	s = cgr_get_sess(ctx, pv_get_idx_value(msg, param));
@@ -572,11 +575,11 @@ static int pv_get_cgr_reply(struct sip_msg *msg, pv_param_t *param,
 	} else {
 		if (param->pvn.type == CGR_PV_NAME_VAR) {
 			if (pv_get_spec_value(msg, (pv_spec_p)param->pvn.u.dname, val) != 0) {
-				LM_ERR("cannot get the name of the $cgrret variable\n");
+				LM_ERR("cannot get the name of the $cgr_ret variable\n");
 				return -1;
 			}
 			if (val->flags & PV_VAL_NULL || !(val->flags & PV_VAL_STR)) {
-				LM_ERR("invalid name for the $cgrret variable!\n");
+				LM_ERR("invalid name for the $cgr_ret variable!\n");
 				return -1;
 			}
 			tmp = val->rs;
@@ -601,7 +604,7 @@ static int pv_get_cgr_reply(struct sip_msg *msg, pv_param_t *param,
 	return 0;
 }
 
-static int pv_parse_cgr(pv_spec_p sp, str *in)
+static int pv_parse_cgr(pv_spec_p sp, const str *in)
 {
 	char *s;
 	pv_spec_t *pv;
@@ -639,7 +642,7 @@ static int pv_parse_cgr(pv_spec_p sp, str *in)
 	return 0;
 }
 
-static int w_pv_parse_cgr(pv_spec_p sp, str *in)
+static int w_pv_parse_cgr(pv_spec_p sp, const str *in)
 {
 	if (cgre_compat_mode) {
 		LM_WARN("using $cgr_opt(%.*s) in compat mode is not possible!\n",
@@ -650,17 +653,7 @@ static int w_pv_parse_cgr(pv_spec_p sp, str *in)
 	return pv_parse_cgr(sp, in);
 }
 
-static int w_pv_parse_cgr_warn(pv_spec_p sp, str *in)
-{
-	static int warned = 0;
-	if (!warned) {
-		LM_WARN("$cgrret(name) is deprecated - please using $cgr_ret(name) instead!\n");
-		warned = 1;
-	}
-	return pv_parse_cgr(sp, in);
-}
-
-static int pv_parse_idx_cgr(pv_spec_p sp, str *in)
+static int pv_parse_idx_cgr(pv_spec_p sp, const str *in)
 {
 	str *s;
 	pv_spec_t *pv;
@@ -679,7 +672,7 @@ static int pv_parse_idx_cgr(pv_spec_p sp, str *in)
 					in->len, in->s);
 			return -1;
 		}
-		sp->pvp.pvi.u.dval = sp;
+		sp->pvp.pvi.u.dval = pv;
 		sp->pvp.pvi.type = CGR_PV_NAME_VAR;
 	} else {
 		/* we need to add the null terminator */
