@@ -327,7 +327,6 @@ int db_sqlite_realloc_rows(db_res_t* res, const unsigned int rows)
 	/* if the values was relocated, we need to re-point all values to the new block,
 	 * otherwise only fix the new ones */
 	start = (res_rows[0].values == prev_values?RES_ROW_N(res):0);
-	start = RES_ROW_N(res);
 
 	for( i=start ; i<rows ; i++ ) {
 		/* the values of the row i */
@@ -375,8 +374,6 @@ static inline int db_sqlite_convert_rows(const db_con_t* _h, db_res_t* _r)
 
 	while (ret != SQLITE_DONE) {
 		ret = sqlite3_step(CON_SQLITE_PS(_h));
-		if (ret == SQLITE_BUSY)
-			continue;
 
 		if (ret == SQLITE_DONE) {
 			RES_ROW_N(_r) = RES_LAST_ROW(_r) = RES_NUM_ROWS(_r) = row;
