@@ -301,7 +301,7 @@ static int l_sipmysql_escape(lua_State *L)
   to = pkg_malloc(2 * len + 1);
   if (!to)
     {
-      siplua_log(L_CRIT, "malloc of %d bytes failed\n", 2 * len + 1);
+      siplua_log(L_CRIT, "malloc of %lu bytes failed\n", 2 * len + 1);
       lua_pushnil(L);
       return 1;
     }
@@ -369,7 +369,7 @@ static int l_sipmysql_prepare(lua_State *L)
 	      o_stmt->bind = pkg_malloc(o_stmt->param_count * sizeof(MYSQL_BIND));
 	      if (!o_stmt->bind)
 		{
-		  siplua_log(L_CRIT, "malloc of %d bytes failed\n",
+		  siplua_log(L_CRIT, "malloc of %lu bytes failed\n",
 			     o_stmt->param_count * sizeof(MYSQL_BIND));
 		  lua_remove(L, -1);
 		  lua_pushnil(L);
@@ -379,7 +379,7 @@ static int l_sipmysql_prepare(lua_State *L)
 	      o_stmt->is_null = pkg_malloc(o_stmt->param_count * sizeof(my_bool));
 	      if (!o_stmt->is_null)
 		{
-		  siplua_log(L_CRIT, "malloc of %d bytes failed\n",
+		  siplua_log(L_CRIT, "malloc of %lu bytes failed\n",
 			     o_stmt->param_count * sizeof(my_bool));
 		  lua_remove(L, -1);
 		  lua_pushnil(L);
@@ -389,7 +389,7 @@ static int l_sipmysql_prepare(lua_State *L)
 	      o_stmt->length = pkg_malloc(o_stmt->param_count * sizeof(unsigned long));
 	      if (!o_stmt->length)
 		{
-		  siplua_log(L_CRIT, "malloc of %d bytes failed\n",
+		  siplua_log(L_CRIT, "malloc of %lu bytes failed\n",
 			     o_stmt->param_count * sizeof(unsigned long));
 		  lua_remove(L, -1);
 		  lua_pushnil(L);
@@ -411,7 +411,7 @@ static int l_sipmysql_prepare(lua_State *L)
 	      o_stmt->result = pkg_malloc(o_stmt->num_fields * sizeof(MYSQL_BIND));
 	      if (!o_stmt->result)
 		{
-		  siplua_log(L_CRIT, "malloc of %d bytes failed\n",
+		  siplua_log(L_CRIT, "malloc of %lu bytes failed\n",
 			     o_stmt->num_fields * sizeof(MYSQL_BIND));
 		  lua_remove(L, -1);
 		  lua_pushnil(L);
@@ -421,7 +421,7 @@ static int l_sipmysql_prepare(lua_State *L)
 	      o_stmt->real_length = pkg_malloc(o_stmt->num_fields * sizeof(unsigned long));
 	      if (!o_stmt->real_length)
 		{
-		  siplua_log(L_CRIT, "malloc of %d bytes failed\n",
+		  siplua_log(L_CRIT, "malloc of %lu bytes failed\n",
 			     o_stmt->num_fields * sizeof(unsigned long));
 		  lua_remove(L, -1);
 		  lua_pushnil(L);
@@ -578,7 +578,7 @@ static int sipmysql_stmt_bind(struct sipmysql_stmt *o, lua_State *L, int n, int
 	o->bind[n].buffer = pkg_malloc(sizeof(number));
 	if (!o->bind[n].buffer)
 	  {
-	    siplua_log(L_CRIT, "malloc of %d bytes failed\n", sizeof(number));
+	    siplua_log(L_CRIT, "malloc of %lu bytes failed\n", sizeof(number));
 	    lua_pushnil(L);
 	    return 1;
 	  }
@@ -597,7 +597,7 @@ static int sipmysql_stmt_bind(struct sipmysql_stmt *o, lua_State *L, int n, int
 	o->bind[n].buffer = pkg_malloc(len);
 	if (!o->bind[n].buffer)
 	  {
-	    siplua_log(L_CRIT, "malloc of %d bytes failed\n", len);
+	    siplua_log(L_CRIT, "malloc of %lu bytes failed\n", len);
 	    lua_pushnil(L);
 	    return 1;
 	  }
@@ -785,7 +785,7 @@ static int sipmysql_stmt_fetch(lua_State *L, int result_type)
 	      buf = pkg_realloc(o->result[i].buffer, o->real_length[i]);
 	      if (!buf)
 		{
-		  siplua_log(L_CRIT, "realloc of %d bytes failed\n", o->real_length[i]);
+		  siplua_log(L_CRIT, "realloc of %lu bytes failed\n", o->real_length[i]);
 		  lua_remove(L, -1);
 		  lua_pushnil(L);
 		  return 1;
