@@ -134,7 +134,8 @@ int end_sub_on_timeout= 1;
 pres_ev_t** pres_event_p= NULL;
 pres_ev_t** dialog_event_p= NULL;
 
-char *federation_mode_str;
+char *federation_mode_str = NULL;
+char *cluster_active_shtag_str = NULL;
 
 int phtable_size= 9;
 phtable_t* pres_htable = NULL;
@@ -150,7 +151,7 @@ static str presence_exposed_event = str_init("E_PRESENCE_EXPOSED");
 event_id_t presence_event_id = EVI_ERROR;
 event_id_t exposed_event_id = EVI_ERROR;
 
-static cmd_export_t cmds[]={
+static const cmd_export_t cmds[]={
 	{"handle_publish",  (cmd_function)handle_publish, {
 		{CMD_PARAM_STR|CMD_PARAM_OPT,fixup_presence,0}, {0,0,0}},
 		REQUEST_ROUTE},
@@ -163,7 +164,7 @@ static cmd_export_t cmds[]={
 	{0,0,{{0,0,0}},0}
 };
 
-static param_export_t params[]={
+static const param_export_t params[]={
 	{ "db_url",                 STR_PARAM, &db_url.s},
 	{ "presentity_table",       STR_PARAM, &presentity_table.s},
 	{ "active_watchers_table",  STR_PARAM, &active_watchers_table.s},
@@ -186,11 +187,12 @@ static param_export_t params[]={
 	{ "end_sub_on_timeout",     INT_PARAM, &end_sub_on_timeout},
 	{ "cluster_id",             INT_PARAM, &pres_cluster_id},
 	{ "cluster_federation_mode",STR_PARAM, &federation_mode_str},
+	{ "cluster_be_active_shtag",STR_PARAM, &cluster_active_shtag_str},
 	{ "cluster_pres_events",    STR_PARAM, &clustering_events.s},
 	{0,0,0}
 };
 
-static mi_export_t mi_cmds[] = {
+static const mi_export_t mi_cmds[] = {
 	// refreshWatchers is a deprecated alias for refresh_watchers. To be removed later.
 	{ "refreshWatchers", 0,0,0, {
 		{mi_refresh_watchers, {"presentity_uri", "event", "refresh_type", 0}},
@@ -221,7 +223,7 @@ static mi_export_t mi_cmds[] = {
 	{EMPTY_MI_EXPORT}
 };
 
-static dep_export_t deps = {
+static const dep_export_t deps = {
 	{ /* OpenSIPS module dependencies */
 		{ MOD_TYPE_DEFAULT, "tm",        DEP_ABORT },
 		{ MOD_TYPE_DEFAULT, "signaling", DEP_ABORT },
@@ -265,7 +267,8 @@ struct module_exports exports= {
 static int mod_init(void)
 {
 	db_url.len = db_url.s ? strlen(db_url.s) : 0;
-	LM_DBG("db_url=%s/%d/%p\n", ZSW(db_url.s), db_url.len,db_url.s);
+	LM_DBG("db_url=%s\n", db_url_escape(&db_url));
+
 	presentity_table.len = strlen(presentity_table.s);
 	active_watchers_table.len = strlen(active_watchers_table.s);
 	watchers_table.len = strlen(watchers_table.s);
@@ -328,18 +331,7 @@ static int mod_init(void)
 		return -1;
 	}
 
-	if (!federation_mode_str || !strcasecmp(federation_mode_str, "disabled")) {
-		cluster_federation = FEDERATION_DISABLED;
-	} else if (!strcasecmp(federation_mode_str, "on-demand-sharing")) {
-		cluster_federation = FEDERATION_ON_DEMAND;
-	} else if (!strcasecmp(federation_mode_str, "full-sharing")) {
-		cluster_federation = FEDERATION_FULL_SHARING;
-	} else {
-		LM_ERR("invalid cluster_federation_mode: '%s'\n", federation_mode_str);
-		return -1;
-	}
-
-	if (init_pres_clustering()<0) {
+	if (init_pres_clustering(federation_mode_str, cluster_active_shtag_str)<0) {
 		LM_ERR("failed to init clustering support\n");
 		return -1;
 	}
@@ -882,7 +874,7 @@ static mi_response_t *mi_list_shtable(const mi_params_t *params, str *from, str
 		lock_get(&subs_htable[i].lock);
 		for (s = subs_htable[i].entries->next; s; s = s->next) {
 			if (from) {
-				/* print subscribtion if "from" and "to" match with given wildcard */
+				/* print subscription if "from" and "to" match with given wildcard */
 				rc = from_to_match_subs(s, &match_from, &match_to, from_w, to_w);
 				if (rc < 0)
 					goto error;
@@ -934,7 +926,7 @@ static mi_response_t *mi_list_shtable_2(const mi_params_t *params,
 	return mi_list_shtable(params, &from, &to);
 }
 
-int pres_update_status(subs_t subs, str reason, db_key_t* query_cols,
+int pres_update_status(subs_t *subs, str reason, db_key_t* query_cols,
         db_val_t* query_vals, int n_query_cols, subs_t** subs_array)
 {
 	static db_ps_t my_del_ps = NULL;
@@ -964,29 +956,29 @@ int pres_update_status(subs_t subs, str reason, db_key_t* query_cols,
 	update_vals[u_reason_col].type= DB_STR;
 	n_update_cols++;
 
-	status= subs.status;
-	if(subs.event->get_auth_status(&subs)< 0)
+	status= subs->status;
+	if(subs->event->get_auth_status(subs)< 0)
 	{
 		LM_ERR( "getting status from rules document\n");
 		return -1;
 	}
-	LM_DBG("subs.status= %d\n", subs.status);
-	if(get_status_str(subs.status)== NULL)
+	LM_DBG("subs.status= %d\n", subs->status);
+	if(get_status_str(subs->status)== NULL)
 	{
-		LM_ERR("wrong status: %d\n", subs.status);
+		LM_ERR("wrong status: %d\n", subs->status);
 		return -1;
 	}
 
-	if(subs.status!= status || reason.len!= subs.reason.len ||
-		(reason.s && subs.reason.s && strncmp(reason.s, subs.reason.s,
+	if(subs->status!= status || reason.len!= subs->reason.len ||
+		(reason.s && subs->reason.s && strncmp(reason.s, subs->reason.s,
 											reason.len)))
 	{
 		/* update in watchers_table */
-		query_vals[q_wuser_col].val.str_val= subs.from_user;
-		query_vals[q_wdomain_col].val.str_val= subs.from_domain;
+		query_vals[q_wuser_col].val.str_val= subs->from_user;
+		query_vals[q_wdomain_col].val.str_val= subs->from_domain;
 
-		update_vals[u_status_col].val.int_val= subs.status;
-		update_vals[u_reason_col].val.str_val= subs.reason;
+		update_vals[u_status_col].val.int_val= subs->status;
+		update_vals[u_reason_col].val.str_val= subs->reason;
 
 		if (pa_dbf.use_table(pa_db, &watchers_table) < 0)
 		{
@@ -996,10 +988,10 @@ int pres_update_status(subs_t subs, str reason, db_key_t* query_cols,
 
 		/* if status is terminated and reason="deactivated",
 		 * delete the record from table */
-		if(subs.status == TERMINATED_STATUS && subs.reason.len==11 &&
-				strncmp(subs.reason.s, "deactivated", 11)==0)
+		if(subs->status == TERMINATED_STATUS && subs->reason.len==11 &&
+				strncmp(subs->reason.s, "deactivated", 11)==0)
 		{
-			CON_PS_REFERENCE(pa_db) = &my_del_ps;
+			CON_SET_CURR_PS(pa_db, &my_del_ps);
 			if(pa_dbf.delete(pa_db, query_cols, 0, query_vals, n_query_cols)<0)
 			{
 				LM_ERR( "in sql delete\n");
@@ -1008,7 +1000,7 @@ int pres_update_status(subs_t subs, str reason, db_key_t* query_cols,
 		}
 		else
 		{
-			CON_PS_REFERENCE(pa_db) = &my_upd_ps;
+			CON_SET_CURR_PS(pa_db, &my_upd_ps);
 			if(pa_dbf.update(pa_db, query_cols, 0, query_vals, update_cols,
 						update_vals, n_query_cols, n_update_cols)< 0)
 			{
@@ -1019,12 +1011,12 @@ int pres_update_status(subs_t subs, str reason, db_key_t* query_cols,
 
 		/* save in the list all affected dialogs */
 		/* if status switches to terminated -> delete dialog */
-		if(update_pw_dialogs(&subs, subs.db_flag, subs_array)< 0)
+		if(update_pw_dialogs(subs, subs->db_flag, subs_array)< 0)
 		{
 			LM_ERR( "extracting dialogs from [watcher]=%.*s@%.*s to"
-				" [presentity]=%.*s\n",	subs.from_user.len, subs.from_user.s,
-				subs.from_domain.len, subs.from_domain.s, subs.pres_uri.len,
-				subs.pres_uri.s);
+				" [presentity]=%.*s\n",	subs->from_user.len, subs->from_user.s,
+				subs->from_domain.len, subs->from_domain.s, subs->pres_uri.len,
+				subs->pres_uri.s);
 			return -1;
 		}
 	}
@@ -1068,8 +1060,7 @@ int pres_db_delete_status(subs_t* s)
 	query_vals[n_query_cols].val.str_val= s->from_domain;
 	n_query_cols++;
 
-	CON_PS_REFERENCE(pa_db) = &my_ps;
-
+	CON_SET_CURR_PS(pa_db, &my_ps);
 	if(pa_dbf.delete(pa_db, query_cols, 0, query_vals, n_query_cols)< 0)
 	{
 		LM_ERR("sql delete failed\n");
@@ -1176,7 +1167,7 @@ int update_watchers_status(str pres_uri, pres_ev_t* ev, str* rules_doc)
 		goto done;
 	}
 
-//	CON_PS_REFERENCE(pa_db) = &my_ps;
+//	CON_SET_CURR_PS(pa_db, &my_ps);
 	if(pa_dbf.query(pa_db, query_cols, 0, query_vals, result_cols,n_query_cols,
 				n_result_cols, 0, &result)< 0)
 	{
@@ -1270,7 +1261,7 @@ int update_watchers_status(str pres_uri, pres_ev_t* ev, str* rules_doc)
 			subs.status = ws_list[i].status;
 			memset(&subs.reason, 0, sizeof(str));
 
-			if( pres_update_status(subs, reason, query_cols, query_vals,
+			if( pres_update_status(&subs, reason, query_cols, query_vals,
 					n_query_cols, &subs_array)< 0)
 			{
 				LM_ERR("failed to update watcher status\n");
@@ -1313,7 +1304,7 @@ int update_watchers_status(str pres_uri, pres_ev_t* ev, str* rules_doc)
 		subs.status= status;
 		memset(&subs.reason, 0, sizeof(str));
 
- 		if( pres_update_status(subs,reason, query_cols, query_vals,
+		if( pres_update_status(&subs,reason, query_cols, query_vals,
 					n_query_cols, &subs_array)< 0)
 		{
 			LM_ERR("failed to update watcher status\n");
@@ -1487,7 +1478,7 @@ static int update_pw_dialogs(subs_t* subs, unsigned int hash_code, subs_t** subs
                 lock_release(&subs_htable[hash_code].lock);
                 return -1;
 			}
-			cs->expires-= (int)time(NULL);
+			cs->expires-= (unsigned int)(unsigned long)time(NULL);
 			cs->next= (*subs_array);
 			(*subs_array)= cs;
 			if(subs->status== TERMINATED_STATUS)
