@@ -29,12 +29,19 @@
 #include "siprec_sess.h"
 #include "../b2b_entities/b2be_load.h"
 
-int src_start_recording(struct sip_msg *msg, struct src_sess *sess);
 void tm_start_recording(struct cell *t, int type, struct tmcb_params *ps);
 int srec_register_callbacks(struct src_sess *sess);
 int srec_restore_callback(struct src_sess *sess);
-void srec_logic_destroy(struct src_sess *sess);
+void srec_logic_destroy(struct src_sess *sess, int keep_sdp);
+void srec_nodes_destroy(struct src_sess *sess);
+int src_pause_recording(str *instance);
+int src_resume_recording(str *instance);
+int srec_late_recording(struct src_sess *ss);
+int src_start_recording(struct sip_msg *msg, struct src_sess *sess);
+int src_send_indialog(struct sip_msg *msg, str *hdrs, str *body, str *instance);
+int src_stop_recording(str *instance);
 
+extern int srec_dlg_idx;
 extern struct b2b_api srec_b2b;
 extern str skip_failover_codes;
 int src_init(void);
