@@ -33,22 +33,10 @@
  *
  */
 
+/* make strptime available */
 #define _GNU_SOURCE
-#define _XOPEN_SOURCE 600          /* glibc2 on linux, bsd */
-#define _XOPEN_SOURCE_EXTENDED 1   /* solaris */
-
-/**
- * _XOPEN_SOURCE creates conflict in swab definition in Solaris
- */
-#ifdef __OS_solaris
-	#undef _XOPEN_SOURCE
-#endif
-
 #include <time.h>
 
-#undef _XOPEN_SOURCE
-#undef _XOPEN_SOURCE_EXTENDED
-
 #include <openssl/x509.h>
 
 #undef _GNU_SOURCE
@@ -70,6 +58,9 @@
 #include "../../data_lump.h"
 #include "../../lib/cJSON.h"
 #include "../../context.h"
+#include "../../mod_fix.h"
+#include "../../data_lump_rpl.h"
+#include "../../mi/mi.h"
 
 #include "stir_shaken.h"
 
@@ -81,11 +72,39 @@
 #define parsed_ctx_set(_ptr) \
 	context_put_ptr(CONTEXT_GLOBAL, current_processing_ctx, parsed_ctx_idx, _ptr)
 
+#define SS_LOCK \
+	do { \
+		if (ss_openssl_lock) \
+			lock_get(ss_openssl_lock); \
+		else \
+			LM_DBG("cannot lock openssl\n"); \
+	} while (0)
+#define SS_UNLOCK \
+	do { \
+		if (ss_openssl_lock) \
+			lock_release(ss_openssl_lock); \
+	} while (0)
+
+/*
+ * Module core functions
+ */
 static int mod_init(void);
 static void mod_destroy(void);
 
+
+/*
+ * Module internal functions
+ */
+static int fixup_auth_out(void** param);
+
+
+/*
+ * Script functions
+ */
 static int w_stir_auth(struct sip_msg *msg, str *attest, str *origid,
-	str *cert_buf, str *pkey_buf, str *cr_url, str *orig_tn_p, str *dest_tn_p);
+	str *cert_buf, str *pkey_buf, str *cr_url, str *orig_tn_p, str *dest_tn_p,
+	struct auth_out_param *out);
+static int w_stir_disengagement(struct sip_msg *msg, str *token);
 static int w_stir_verify(struct sip_msg *msg, str *cert_buf,
 	pv_spec_t *err_code, pv_spec_t *err_reason, str *orig_tn_p, str *dest_tn_p);
 static int w_stir_check(struct sip_msg *msg);
@@ -93,9 +112,23 @@ static int w_stir_check_cert(struct sip_msg *msg, str *cert_buf);
 static int fixup_attest(void **param);
 static int fixup_check_wrvar(void **param);
 
+/*
+ * Exported Pseudo-Variables
+ */
 int pv_get_identity(struct sip_msg *msg, pv_param_t *param, pv_value_t *res);
-int pv_parse_identity_name(pv_spec_p sp, str *in);
+int pv_parse_identity_name(pv_spec_p sp, const str *in);
 
+
+/*
+ * MI functions
+ */
+mi_response_t *mi_stir_shaken_ca_reload(const mi_params_t *params, struct mi_handler *async_hdl);
+mi_response_t *mi_stir_shaken_crl_reload(const mi_params_t *params, struct mi_handler *async_hdl);
+
+
+/*
+ * Module internal parameter variables
+ */
 static int auth_date_freshness = DEFAULT_AUTH_FRESHNESS;
 static int verify_date_freshness = DEFAULT_VERIFY_FRESHNESS;
 static char *ca_list;
@@ -104,14 +137,19 @@ static char *crl_list;
 static char *crl_dir;
 
 static int e164_strict_mode;
+static int e164_max_length = 15;
+
+static int require_date_hdr = 1;
 
 static int tn_authlist_nid;
 
 static int parsed_ctx_idx =-1;
 
 static X509_STORE *store;
+static gen_lock_t *ss_openssl_lock;
 
-static param_export_t params[] = {
+
+static const param_export_t params[] = {
 	{"auth_date_freshness", INT_PARAM, &auth_date_freshness},
 	{"verify_date_freshness", INT_PARAM, &verify_date_freshness},
 	{"ca_list", STR_PARAM, &ca_list},
@@ -119,16 +157,18 @@ static param_export_t params[] = {
 	{"crl_list", STR_PARAM, &crl_list},
 	{"crl_dir", STR_PARAM, &crl_dir},
 	{"e164_strict_mode", INT_PARAM, &e164_strict_mode},
+	{"e164_max_length", INT_PARAM, &e164_max_length},
+	{"require_date_hdr", INT_PARAM, &require_date_hdr},
 	{0, 0, 0}
 };
 
-static pv_export_t mod_items[] = {
-	{{"identity", sizeof("identity") - 1}, 1000, pv_get_identity,
+static const pv_export_t mod_items[] = {
+	{str_const_init("identity"), 1000, pv_get_identity,
 		0, pv_parse_identity_name, 0, 0, 0},
 	{ {0, 0}, 0, 0, 0, 0, 0, 0, 0 }
 };
 
-static cmd_export_t cmds[] = {
+static const cmd_export_t cmds[] = {
 	{"stir_shaken_auth", (cmd_function)w_stir_auth, {
 		{CMD_PARAM_STR, fixup_attest, 0},
 		{CMD_PARAM_STR, 0, 0},
@@ -136,8 +176,13 @@ static cmd_export_t cmds[] = {
 		{CMD_PARAM_STR, 0, 0},
 		{CMD_PARAM_STR, 0, 0},
 		{CMD_PARAM_STR|CMD_PARAM_OPT, 0, 0},
-		{CMD_PARAM_STR|CMD_PARAM_OPT, 0, 0}, {0,0,0}},
+		{CMD_PARAM_STR|CMD_PARAM_OPT, 0, 0},
+		{CMD_PARAM_STR|CMD_PARAM_OPT|CMD_PARAM_NO_EXPAND,
+			fixup_auth_out, fixup_free_pkg}, {0,0,0}},
 		REQUEST_ROUTE},
+	{"stir_shaken_disengagement", (cmd_function)w_stir_disengagement, {
+		{CMD_PARAM_STR, 0, 0},
+		{0,0,0}}, REQUEST_ROUTE},
 	{"stir_shaken_verify", (cmd_function)w_stir_verify, {
 		{CMD_PARAM_STR, 0, 0},
 		{CMD_PARAM_VAR, fixup_check_wrvar, 0},
@@ -153,6 +198,25 @@ static cmd_export_t cmds[] = {
 	{0,0,{{0,0,0}},0}
 };
 
+/*
+ * Exported MI functions
+ */
+static const mi_export_t mi_cmds[] = {
+	{ "stir_shaken_ca_reload", "Stir shaken ca reloader", 0, 0, {
+		{mi_stir_shaken_ca_reload, {0}},
+		{EMPTY_MI_RECIPE}}
+	},
+	{ "stir_shaken_crl_reload", "Stir shaken crl reloader", 0, 0, {
+		{mi_stir_shaken_crl_reload, {0}},
+		{EMPTY_MI_RECIPE}}
+	},
+	{EMPTY_MI_EXPORT}
+};
+
+
+/*
+ * Module interface
+ */
 struct module_exports exports = {
 	"stir_shaken",    /* module name*/
 	MOD_TYPE_DEFAULT, /* class of this module */
@@ -164,7 +228,7 @@ struct module_exports exports = {
 	0,          /* exported async functions */
 	params,     /* module parameters */
 	0,          /* exported statistics */
-	0,          /* exported MI functions */
+	mi_cmds,	/* exported MI functions */
 	mod_items,  /* exported pseudo-variables */
 	0,			/* exported transformations */
 	0,          /* extra processes */
@@ -190,6 +254,7 @@ static int verify_callback(int ok, X509_STORE_CTX *ctx)
 	return ok;
 }
 
+// called during mod_init
 static int init_cert_validation(void)
 {
 	store = X509_STORE_new();
@@ -201,11 +266,11 @@ static int init_cert_validation(void)
 
 	if (ca_list || ca_dir) {
 		if (X509_STORE_load_locations(store, ca_list, ca_dir) != 1) {
-			LM_ERR("Failed to load trustefd CAs\n");
+			LM_ERR("Failed to load trusted CAs\n");
 			return -1;
 		}
 		if (X509_STORE_set_default_paths(store) != 1) {
-			LM_ERR("Failed to loade the system-wide CA certificates\n");
+			LM_ERR("Failed to load the system-wide CA certificates\n");
 			return -1;
 		}
 	}
@@ -219,6 +284,83 @@ static int init_cert_validation(void)
 			X509_V_FLAG_CRL_CHECK | X509_V_FLAG_CRL_CHECK_ALL);
 	}
 
+	ss_openssl_lock = lock_alloc();
+	if (!ss_openssl_lock || !lock_init(ss_openssl_lock)) {
+		LM_ERR("could not allocate openssl lock\n");
+		ss_openssl_lock = NULL;
+	}
+
+	return 0;
+}
+
+// store check then reload ca_list and ca_dir
+static int init_cert_ca_reload(void)
+{
+	store = X509_STORE_new();
+	if (!store) {
+		LM_ERR("Failed to create X509_STORE_CTX object\n");
+		return -1;
+	}
+	X509_STORE_set_verify_cb_func(store, verify_callback);
+
+	/* check if ca_list param is set */
+	if (!ca_list) {
+		LM_ERR("Failed, path to a file containing trusted CA certificates for the verifier is not set or the certificates must be in PEM format, one after another.\n");
+		return -2;
+	}
+
+	/* check if ca_dir param is set */
+	if (!ca_dir) {
+		LM_ERR("Failed, path to a directory containing trusted CA certificates for the verifier is not set or the certificates in the directory must be in hashed form, as described in the openssl documentation for the Hashed Directory Method.\n");
+		return -3;
+	}
+
+	/* params are set, do reload */
+	if (ca_list || ca_dir) {
+		if (X509_STORE_load_locations(store, ca_list, ca_dir) != 1) {
+			LM_ERR("Failed to load trusted CAs\n");
+			return -4;
+		}
+		if (X509_STORE_set_default_paths(store) != 1) {
+			LM_ERR("Failed to load the system-wide CA certificates\n");
+			return -5;
+		}
+	}
+
+	return 0;
+}
+
+// store check then reload crl_list and crl_dir
+static int init_cert_crl_reload(void)
+{
+	store = X509_STORE_new();
+	if (!store) {
+		LM_ERR("Failed to create X509_STORE_CTX object\n");
+		return -1;
+	}
+	X509_STORE_set_verify_cb_func(store, verify_callback);
+
+	/* check if crl_list param is set */
+	if (!crl_list) {
+		LM_ERR("Failed, path to a file containing certificate revocation lists (CRLs) for the verifier is not set.\n");
+		return -2;
+	}
+
+	/* check if crl_dir param is set */
+	if (!crl_dir) {
+		LM_ERR("Failed, path to a directory containing certificate revocation lists (CRLs) for the verifier is not set or the CRLs in the directory must be in hashed form, as described in the openssl documentation for the Hashed Directory Method.\n");
+		return -3;
+	}
+
+	/* params are set, do reload */
+	if (crl_list || crl_dir) {
+		if (X509_STORE_load_locations(store, crl_list, crl_dir) != 1) {
+			LM_ERR("Failed to load CRLs\n");
+			return -4;
+		}
+		X509_STORE_set_flags(store, X509_V_FLAG_CRL_CHECK | X509_V_FLAG_CRL_CHECK_ALL);
+	}
+
 	return 0;
 }
 
@@ -331,12 +473,13 @@ static int add_date_hf(struct sip_msg *msg, time_t *date_ts)
 	#define DATE_HDR_S  "Date: "
 	#define DATE_HDR_L  (sizeof(DATE_HDR_S)-1)
 
+	struct tm ldate_tm;
 	struct tm *date_tm;
 	char *buf;
 	int len;
 	struct lump* anchor;
 
-	date_tm = gmtime(date_ts);
+	date_tm = gmtime_r(date_ts, &ldate_tm);
 	if (!date_tm) {
 		LM_ERR("Failed to convert timestamp to broken-down time\n");
 		return -1;
@@ -371,6 +514,62 @@ static int add_date_hf(struct sip_msg *msg, time_t *date_ts)
 	return 0;
 }
 
+/* Add P-Identity-Bypass for Incident Management Response */
+static int add_disengagement_token(struct sip_msg *msg, str *token)
+{
+
+	#define DISENGAGEMENT_HDR_S		"P-Identity-Bypass: "
+	#define DISENGAGEMENT_HDR_L		sizeof(DISENGAGEMENT_HDR_S)
+	
+	char *buf;
+	unsigned int len;
+	struct lump* anchor;
+
+	/* make sure we detect all headers */
+	if (parse_headers(msg, HDR_EOH_F, 0) == -1) {
+		LM_ERR("error while parsing message\n");
+		return -1;
+	}
+
+	/* check if token is available */
+	if (!token){
+		LM_ERR("Failed to identify disengagement token\n");
+		return -1;
+	}
+
+	/* add the anchor at the very end of the SIP headers */
+	anchor = anchor_lump(msg, msg->unparsed - msg->buf, 0);
+	if (anchor == NULL) {
+		LM_ERR("Failed to create lump anchor\n");
+		return -1;
+	}
+
+	/* calculate len (header + token + crlf) */
+	len = strlen(DISENGAGEMENT_HDR_S) + strlen(token->s) + strlen(CRLF);
+	/* alloc pkg memory */
+	buf= pkg_malloc(len);
+	if (!buf) {
+		LM_ERR("No more pkg mem\n");
+		return -1;
+	}
+
+	// push header in the buffer
+	strcpy(buf, DISENGAGEMENT_HDR_S);
+	// push token after the header
+	strcat(buf, token->s);
+	// push "\r\n" after the token
+	strcat(buf, CRLF);
+
+	/* insert buf at the end of previous headers */
+	if (insert_new_lump_after(anchor, buf, len, 0) == 0) {
+		LM_ERR("can't insert lump\n");
+		pkg_free(buf);
+		return -1;
+	}
+
+	return 0;
+}
+
 static int check_cert_validity(time_t *timestamp, X509 *cert)
 {
 	ASN1_STRING *notBeforeSt;
@@ -691,16 +890,15 @@ void ECDSA_SIG_get0(const ECDSA_SIG *sig, const BIGNUM **pr, const BIGNUM **ps)
 }
 #endif
 
-static int add_identity_hf(struct sip_msg *msg, EVP_PKEY *pkey,
+static str *build_identity_hf(EVP_PKEY *pkey,
 	time_t date_ts, str *attest, str *cr_url, str *orig_tn,
 	str *dest_tn, str *origid)
 {
-	str hdr_buf = {0,0};
+	static str hdr_buf = {0,0};
 	str unsigned_buf;
 	str der_sig_buf = {0,0};
 	unsigned char *der_sig_p;
 	unsigned char raw_sig_buf[RAW_SIG_LEN];
-	struct lump* anchor;
 	EVP_MD_CTX *mdctx = NULL;
 	ECDSA_SIG *sig = NULL;
 	const BIGNUM *r, *s;
@@ -709,7 +907,7 @@ static int add_identity_hf(struct sip_msg *msg, EVP_PKEY *pkey,
 	if (build_unsigned_pport(&unsigned_buf, date_ts, attest, cr_url,
 		orig_tn, dest_tn, origid) < 0) {
 		LM_ERR("Failed to build PASSporT\n");
-		return -1;
+		return NULL;
 	}
 
 	mdctx = EVP_MD_CTX_create();
@@ -775,7 +973,7 @@ static int add_identity_hf(struct sip_msg *msg, EVP_PKEY *pkey,
 
 	hdr_buf.len = IDENTITY_HDR_LEN + unsigned_buf.len + 1/*'.'*/ +
 		calc_base64_encode_len(RAW_SIG_LEN) + 1/*';'*/ + HDR_INFO_PARAM_LEN +
-		2/*'<','>'*/ + cr_url->len + 1/*';'*/ + HDR_PPT_PARAM_LEN + CRLF_LEN;
+		2/*'<','>'*/ + cr_url->len + 1/*';'*/ + HDR_ALG_PARAM_LEN + 1/*';'*/ + HDR_PPT_PARAM_LEN + CRLF_LEN;
 	hdr_buf.s = pkg_malloc(hdr_buf.len);
 	if (!hdr_buf.s) {
 		LM_ERR("oom!\n");
@@ -805,22 +1003,15 @@ static int add_identity_hf(struct sip_msg *msg, EVP_PKEY *pkey,
 	hdr_buf.len += cr_url->len;
 	hdr_buf.s[hdr_buf.len++] = '>';
 	hdr_buf.s[hdr_buf.len++] = ';';
+	memcpy(hdr_buf.s + hdr_buf.len, HDR_ALG_PARAM_S, HDR_ALG_PARAM_LEN);
+        hdr_buf.len += HDR_ALG_PARAM_LEN;
+        hdr_buf.s[hdr_buf.len++] = ';';
 	memcpy(hdr_buf.s + hdr_buf.len, HDR_PPT_PARAM_S, HDR_PPT_PARAM_LEN);
 	hdr_buf.len += HDR_PPT_PARAM_LEN;
 	memcpy(hdr_buf.s + hdr_buf.len, CRLF, CRLF_LEN);
 	hdr_buf.len += CRLF_LEN;
 
-	anchor = anchor_lump(msg, msg->unparsed - msg->buf, 0);
-	if (!anchor) {
-		LM_ERR("Failed to get anchor lump\n");
-		goto error;
-	}
-	if (!insert_new_lump_before(anchor, hdr_buf.s, hdr_buf.len, 0)) {
-		LM_ERR("Failed to insert lump\n");
-		goto error;
-	}
-
-	return 0;
+	return &hdr_buf;
 
 error:
 	pkg_free(unsigned_buf.s);
@@ -832,7 +1023,8 @@ error:
 		pkg_free(der_sig_buf.s);
 	if (sig)
 		ECDSA_SIG_free(sig);
-	return -1;
+
+	return NULL;
 }
 
 static int load_cert(X509 **cert, STACK_OF(X509) **certchain, str *cert_buf)
@@ -847,12 +1039,14 @@ static int load_cert(X509 **cert, STACK_OF(X509) **certchain, str *cert_buf)
 		LM_ERR("Unable to create BIO buf\n");
 		return -1;
 	}
+	SS_LOCK;
 
 	/* parse end-entity certificate */
 	*cert = PEM_read_bio_X509(cbio, NULL, 0, NULL);
 	if (*cert == NULL) {
 		LM_ERR("Failed to parse certificate\n");
 		BIO_free(cbio);
+		SS_UNLOCK;
 		return -1;
 	}
 
@@ -864,6 +1058,7 @@ static int load_cert(X509 **cert, STACK_OF(X509) **certchain, str *cert_buf)
 			X509_free(*cert);
 			*cert = NULL;
 			BIO_free(cbio);
+			SS_UNLOCK;
 			return -1;
 		}
 
@@ -874,6 +1069,7 @@ static int load_cert(X509 **cert, STACK_OF(X509) **certchain, str *cert_buf)
 			*cert = NULL;
 			BIO_free(cbio);
 			sk_X509_free(stack);
+			SS_UNLOCK;
 			return -1;
 		}
 
@@ -896,6 +1092,7 @@ static int load_cert(X509 **cert, STACK_OF(X509) **certchain, str *cert_buf)
 	} else {
 		BIO_free(cbio);
 	}
+	SS_UNLOCK;
 
 	return 0;
 }
@@ -910,14 +1107,17 @@ static int load_pkey(EVP_PKEY **pkey, str *pkey_buf)
 		return -1;
 	}
 
+	SS_LOCK;
 	*pkey = PEM_read_bio_PrivateKey(kbio, NULL, NULL, NULL);
 	if (*pkey == NULL) {
 		LM_ERR("Failed to load private key from buffer\n");
 		BIO_free(kbio);
+		SS_UNLOCK;
 		return -1;
 	}
 
 	BIO_free(kbio);
+	SS_UNLOCK;
 
 	return 0;
 }
@@ -936,7 +1136,7 @@ static int check_passport_phonenum(str *num, int log_lev)
 		num->len--;
 	}
 
-	if (_is_e164(num, e164_strict_mode) == -1) {
+	if (_is_e164(num, e164_strict_mode, e164_max_length) == -1) {
 		LM_GEN(log_lev, "number is not in E.164 format: %.*s\n", num->len, num->s);
 		return -1;
 	}
@@ -944,8 +1144,39 @@ static int check_passport_phonenum(str *num, int log_lev)
 	return 0;
 }
 
+static int fixup_auth_out(void** param)
+{
+	struct auth_out_param *out_p;
+	str *s = (str*)*param;
+
+	out_p = pkg_malloc(sizeof *out_p);
+	memset(out_p, 0, sizeof *out_p);
+
+	if (!str_strcmp(_str(AUTH_OUT_REQ_STR), s)) {
+		out_p->type = AUTH_APPEND_TO_REQ;
+	} else if (!str_strcmp(_str(AUTH_OUT_RPL_STR), s)) {
+		out_p->type = AUTH_APPEND_TO_RPL;
+	} else if (*s->s == PV_MARKER) {
+		out_p->type = AUTH_OUT_VAR;
+
+		if (pv_parse_spec(s, &out_p->var) == NULL) {
+			pkg_free(out_p);
+			LM_ERR("Failed to parse output variable spec\n");
+			return -1;
+		}
+	} else {
+		LM_ERR("Expected variable or the 'req'/'rpl' flags\n");
+		return -1;
+	}
+
+	*param = out_p;
+
+	return 0;
+}
+
 static int w_stir_auth(struct sip_msg *msg, str *attest, str *origid,
-	str *cert_buf, str *pkey_buf, str *cr_url, str *orig_tn_p, str *dest_tn_p)
+	str *cert_buf, str *pkey_buf, str *cr_url, str *orig_tn_p, str *dest_tn_p,
+	struct auth_out_param *out_p)
 {
 	time_t now, date_ts;
 	struct hdr_field *date_hf = NULL;
@@ -953,6 +1184,9 @@ static int w_stir_auth(struct sip_msg *msg, str *attest, str *origid,
 	EVP_PKEY *pkey = NULL;
 	str orig_tn, dest_tn;
 	int rc, orig_log_lev = L_ERR, dest_log_lev = L_ERR;
+	str *hdr_buf = NULL;
+	struct lump* anchor;
+	pv_value_t out_val;
 
 	/* looking for 'Identity' and 'Date' */
 	if (parse_headers(msg, HDR_EOH_F, 0) < 0) {
@@ -960,11 +1194,6 @@ static int w_stir_auth(struct sip_msg *msg, str *attest, str *origid,
 		return -1;
 	}
 
-	if (get_header_by_static_name(msg, "Identity")) {
-		LM_NOTICE("Identity header already exists\n");
-		return -2;
-	}
-
 	if (!orig_tn_p) {
 		if ((rc = get_orig_tn_from_msg(msg, &orig_tn)) < 0) {
 			if (rc == -1)
@@ -1021,9 +1250,10 @@ static int w_stir_auth(struct sip_msg *msg, str *attest, str *origid,
 			return -1;
 		}
 
-		if (now - date_ts > auth_date_freshness) {
-			LM_NOTICE("Date header value is older than local policy "
-			          "(%lds > %ds)\n", now - date_ts, auth_date_freshness);
+		if (labs(now - date_ts) > auth_date_freshness) {
+			LM_NOTICE("Date header timestamp diff exceeds local policy "
+			    "(diff: %llds, auth-freshness: %ds)\n",
+			    (long long)(now - date_ts), auth_date_freshness);
 			return -4;
 		}
 	}
@@ -1049,24 +1279,65 @@ static int w_stir_auth(struct sip_msg *msg, str *attest, str *origid,
 		goto error;
 	}
 
-	if (add_identity_hf(msg, pkey, date_ts, attest, cr_url,
-		orig_tn_p, dest_tn_p, origid) < 0) {
+	if ((hdr_buf = build_identity_hf(pkey, date_ts, attest, cr_url,
+		orig_tn_p, dest_tn_p, origid)) ==  NULL) {
 		LM_ERR("Failed to add Identity header\n");
 		rc = -1;
 		goto error;
 	}
 
+	if (!out_p || out_p->type == AUTH_APPEND_TO_REQ) {
+		anchor = anchor_lump(msg, msg->unparsed - msg->buf, 0);
+		if (!anchor) {
+			LM_ERR("Failed to get anchor lump\n");
+			rc = -1;
+			goto error;
+		}
+		if (!insert_new_lump_before(anchor, hdr_buf->s, hdr_buf->len, 0)) {
+			LM_ERR("Failed to insert lump\n");
+			rc = -1;
+			goto error;
+		}
+	} else if (out_p->type == AUTH_APPEND_TO_RPL) {
+		if (!add_lump_rpl(msg, hdr_buf->s, hdr_buf->len, LUMP_RPL_HDR)) {
+			LM_ERR("unable to add reply lump\n");
+			rc = -1;
+			goto error;
+		}
+	} else {
+		out_val.flags = PV_VAL_STR;
+		out_val.rs = *hdr_buf;
+		if (pv_set_value(msg, &out_p->var, 0, &out_val) != 0) {
+			rc = -1;
+			goto error;
+		}
+
+		pkg_free(hdr_buf->s);
+	}
+
 	X509_free(cert);
 	EVP_PKEY_free(pkey);
 
 	return 1;
 error:
+	if (hdr_buf)
+		pkg_free(hdr_buf->s);
 	X509_free(cert);
 	if (pkey)
 		EVP_PKEY_free(pkey);
 	return rc;
 }
 
+/* stir shaken disengagement token function */
+static int w_stir_disengagement(struct sip_msg *msg, str *token)
+{
+	if (add_disengagement_token(msg, token) < 0) {
+		LM_ERR("Failed to add P-Identity-Bypass header\n");
+		return 0;
+	}
+	return 1;
+}
+
 /* decode base64url without padding
  * the actual buffer _in points to is assumed to
  * have a size of at least _in->len+2 */
@@ -1160,7 +1431,7 @@ static int parse_identity_hf(str *hdr_buf, struct parsed_identity *parsed)
 	str header_str, payload_str, sig_str, params_str;
 	char *p;
 	param_hooks_t _;
-	param_t* params = NULL;
+	param_t* params = NULL, *it;
 	int rc = -1;
 
 	payload_str.s = q_memchr(hdr_buf->s, PPORT_SEPARATOR, hdr_buf->len);
@@ -1212,13 +1483,11 @@ static int parse_identity_hf(str *hdr_buf, struct parsed_identity *parsed)
 		LM_INFO("Header parameters missing\n");
 		goto invalid_hdr;
 	}
-	while (params) {
-		if (!str_strcmp(_str("alg"), &params->name))
-			parsed->alg_hdr_param = params->body;
-		if (!str_strcmp(_str("ppt"), &params->name))
-			parsed->ppt_hdr_param = params->body;
-
-		params = params->next;
+	for (it = params; it; it = it->next) {
+		if (!str_strcmp(const_str("alg"), &it->name))
+			parsed->alg_hdr_param = it->body;
+		if (!str_strcmp(const_str("ppt"), &it->name))
+			parsed->ppt_hdr_param = it->body;
 	}
 
 	parsed->dec_header.len = calc_max_base64_decode_len(header_str.len);
@@ -1469,6 +1738,7 @@ static int verify_signature(X509 *cert,
 	if (parsed->dec_signature.len != RAW_SIG_LEN) {
 		LM_ERR("Bad raw signature length [%d], should be [%d]\n",
 			parsed->dec_signature.len, RAW_SIG_LEN);
+		rc = 0;
 		goto error;
 	}
 
@@ -1584,17 +1854,36 @@ error:
 	return rc;
 }
 
-static int get_parsed_identity(struct hdr_field *identity_hdr,
+static int get_parsed_identity(struct sip_msg *msg,
 	struct parsed_identity **parsed)
 {
-	int rc = 0;
+	struct hdr_field *identity_hdr;
+	int rc;
 
 	*parsed = parsed_ctx_get();
-	if (*parsed == NULL) {
-		if (!current_processing_ctx) {
-			LM_ERR("no processing ctx found!\n");
-			return -1;
-		}
+	if (*parsed)
+		/* we are lucky, the parsing was already done */
+		return 0;
+
+	if (!current_processing_ctx) {
+		LM_ERR("no processing ctx found!\n");
+		return -1;
+	}
+
+	/* look for the Identity hdr */
+	if (parse_headers(msg, HDR_EOH_F, 0) < 0) {
+		LM_ERR("Failed to parse headers\n");
+		return -1;
+	}
+
+	if (!(identity_hdr = get_header_by_static_name(msg, "Identity"))) {
+		LM_INFO("No Identity header found\n");
+		return -2;
+	}
+
+	rc = -2;
+
+	do {
 
 		*parsed = pkg_malloc(sizeof **parsed);
 		if (*parsed == NULL) {
@@ -1604,11 +1893,27 @@ static int get_parsed_identity(struct hdr_field *identity_hdr,
 		memset(*parsed, 0, sizeof **parsed);
 
 		rc = parse_identity_hf(&identity_hdr->body, *parsed);
-		if (rc == 0)
-			parsed_ctx_set(*parsed);
-		else
-			pkg_free(*parsed);
-	}
+		if (rc >= 0) {
+			if (str_strcmp(&(*parsed)->ppt_hdr_param, const_str(PPORT_HDR_PPT_VAL)) == 0)
+				break;
+			LM_INFO("Unsupported '%.*s' extension\n",
+				(*parsed)->ppt_hdr_param.len, (*parsed)->ppt_hdr_param.s);
+			rc = -2; /* consider we did not find a proper Identity header */
+		}
+		pkg_free(*parsed);
+		*parsed = NULL;
+		/* let's check other Identity hdr, if present */
+		identity_hdr = get_next_header_by_static_name ( identity_hdr,
+			"Identity");
+		if (identity_hdr==NULL) {
+			LM_INFO("No valid Identity header found\n");
+			return rc;
+		}
+
+	}while(rc < 0);
+
+	if (rc >= 0)
+		parsed_ctx_set(*parsed);
 
 	return rc;
 }
@@ -1645,7 +1950,6 @@ static int w_stir_verify(struct sip_msg *msg, str *cert_buf,
 	pv_spec_t *err_code_var, pv_spec_t *err_reason_var,
 	str *orig_tn_p, str *dest_tn_p)
 {
-	struct hdr_field *identity_hdr;
 	str orig_tn, dest_tn, pport_orig_tn, pport_dest_tn;
 	time_t now, date_ts, iat_ts;
 	struct hdr_field *date_hf = NULL;
@@ -1655,19 +1959,6 @@ static int w_stir_verify(struct sip_msg *msg, str *cert_buf,
 	int rc, err_code, orig_log_lev = L_ERR, dest_log_lev = L_ERR;
 	char *err_reason;
 
-	/* looking for 'Identity' and 'Date' */
-	if (parse_headers(msg, HDR_EOH_F, 0) < 0) {
-		LM_ERR("Failed to parse headers\n");
-		SET_VERIFY_ERR_VARS(IERROR_CODE, IERROR_REASON);
-		return -1;
-	}
-
-	if (!(identity_hdr = get_header_by_static_name(msg, "Identity"))) {
-		LM_NOTICE("No Identity header found\n");
-		SET_VERIFY_ERR_VARS(USE_IDENTITY_CODE, USE_IDENTITY_REASON);
-		return -2;
-	}
-
 	if (!orig_tn_p) {
 		err_code = BADREQ_CODE;
 		err_reason = BADREQ_ORIG_REASON;
@@ -1721,10 +2012,10 @@ static int w_stir_verify(struct sip_msg *msg, str *cert_buf,
 		return -3;
 	}
 
-	if ((rc = get_parsed_identity(identity_hdr, &parsed)) < 0) {
+	if ((rc = get_parsed_identity( msg, &parsed)) < 0) {
 		if (rc == -1) {
 			LM_ERR("Failed to parse identity header\n");
-			SET_VERIFY_ERR_VARS(IERROR_CODE, IERROR_REASON);
+			SET_VERIFY_ERR_VARS(IERROR_CODE, IERROR_REASON_PARSE_IDENTITY);
 		} else {  /* rc == -4 */
 			LM_INFO("Invalid identity header\n");
 			SET_VERIFY_ERR_VARS(INVALID_IDENTITY_CODE, INVALID_IDENTITY_REASON);
@@ -1733,15 +2024,8 @@ static int w_stir_verify(struct sip_msg *msg, str *cert_buf,
 		return rc;
 	}
 
-	if (str_strcmp(&parsed->ppt_hdr_param, _str(PPORT_HDR_PPT_VAL))) {
-		LM_NOTICE("Unsupported 'ppt' extension: %.*s\n",
-		          parsed->ppt_hdr_param.len, parsed->ppt_hdr_param.s);
-		SET_VERIFY_ERR_VARS(INVALID_IDENTITY_CODE, INVALID_IDENTITY_REASON);
-		rc = -5;
-		goto error;
-	}
 	if (parsed->alg_hdr_param.s &&
-		str_strcmp(&parsed->alg_hdr_param, _str(PPORT_HDR_ALG_VAL))) {
+		str_strcmp(&parsed->alg_hdr_param, const_str(PPORT_HDR_ALG_VAL))) {
 		LM_NOTICE("Unsupported 'alg': %.*s\n",
 		          parsed->alg_hdr_param.len, parsed->alg_hdr_param.s);
 		SET_VERIFY_ERR_VARS(INVALID_IDENTITY_CODE, INVALID_IDENTITY_REASON);
@@ -1757,32 +2041,48 @@ static int w_stir_verify(struct sip_msg *msg, str *cert_buf,
 	}
 
 	date_hf = get_header_by_static_name(msg, "Date");
-	if (!date_hf) {
-		LM_NOTICE("No Date header found\n");
-		SET_VERIFY_ERR_VARS(BADREQ_CODE, BADREQ_NODATE_REASON);
-		rc = -2;
-		goto error;
-	}
-
-	if (get_date_ts(date_hf, &date_ts) < 0) {
-		LM_ERR("Failed to get UNIX time from Date header\n");
-		SET_VERIFY_ERR_VARS(IERROR_CODE, IERROR_REASON);
-		rc = -1;
-		goto error;
-	}
 
 	if ((now = time(0)) == -1) {
 		LM_ERR("Failed to get current time\n");
-		SET_VERIFY_ERR_VARS(IERROR_CODE, IERROR_REASON);
+		SET_VERIFY_ERR_VARS(IERROR_CODE, IERROR_REASON_GET_CURRENT_TIME);
 		rc = -1;
 		goto error;
 	}
-	if (now - date_ts > verify_date_freshness) {
-		LM_NOTICE("Date header value is older than local policy (%lds > %ds)\n",
-		          now - date_ts, verify_date_freshness);
-		SET_VERIFY_ERR_VARS(STALE_DATE_CODE, STALE_DATE_REASON);
-		rc = -6;
-		goto error;
+
+	iat_ts = (time_t)parsed->iat->valuedouble;
+
+	if (require_date_hdr || date_hf) {
+		if (!date_hf) {
+			LM_NOTICE("No Date header found\n");
+			SET_VERIFY_ERR_VARS(BADREQ_CODE, BADREQ_NODATE_REASON);
+			rc = -2;
+			goto error;
+		}
+
+		if (get_date_ts(date_hf, &date_ts) < 0) {
+			LM_ERR("Failed to get UNIX time from Date header\n");
+			SET_VERIFY_ERR_VARS(IERROR_CODE, IERROR_REASON_GET_TIME_FROM_DATE_HEADER);
+			rc = -1;
+			goto error;
+		}
+
+		if (labs(now - date_ts) > verify_date_freshness) {
+			LM_NOTICE("Date header timestamp diff exceeds local policy "
+			    "(diff: %llds, verify-freshness: %ds)\n",
+			    (long long)(now - date_ts), verify_date_freshness);
+			SET_VERIFY_ERR_VARS(STALE_DATE_CODE, STALE_DATE_REASON);
+			rc = -6;
+			goto error;
+		}
+	} else {
+		if (labs(now - iat_ts) > verify_date_freshness) {
+			LM_NOTICE("'iat' timestamp diff exceeds local policy "
+			    "(diff: %llds, verify-freshness: %ds)\n",
+			    (long long)(now - iat_ts), verify_date_freshness);
+			SET_VERIFY_ERR_VARS(STALE_DATE_CODE, STALE_DATE_REASON);
+			rc = -6;
+			goto error;
+		}
 	}
 
 	/* if the identities in the PASSporT and SIP message are different
@@ -1811,22 +2111,31 @@ static int w_stir_verify(struct sip_msg *msg, str *cert_buf,
 
 	if (load_cert(&cert, &certchain, cert_buf) < 0) {
 		LM_ERR("Failed to load certificate\n");
-		SET_VERIFY_ERR_VARS(IERROR_CODE, IERROR_REASON);
+		SET_VERIFY_ERR_VARS(IERROR_CODE, IERROR_REASON_LOAD_CERTIFICATE);
 		rc = -1;
 		goto error;
 	}
 
-	if (!check_cert_validity(&date_ts, cert)) {
-		LM_INFO("The Date header does not fall within the certificate validity\n");
-		SET_VERIFY_ERR_VARS(STALE_DATE_CODE, STALE_DATE_REASON);
-		rc = -7;
-		goto error;
+	if (require_date_hdr || date_hf) {
+		if (!check_cert_validity(&date_ts, cert)) {
+			LM_INFO("The Date header does not fall within the certificate validity\n");
+			SET_VERIFY_ERR_VARS(INVALID_IDENTITY_CODE, INVALID_IDENTITY_REASON " (cert validity)");
+			rc = -7;
+			goto error;
+		}
+	} else {
+		if (!check_cert_validity(&iat_ts, cert)) {
+			LM_INFO("The 'iat' value does not fall within the certificate validity\n");
+			SET_VERIFY_ERR_VARS(INVALID_IDENTITY_CODE, INVALID_IDENTITY_REASON " (cert validity)");
+			rc = -7;
+			goto error;
+		}
 	}
 
 	if ((rc = validate_certificate(cert, certchain)) < 0) {
 		if (rc == -1) {
 			LM_ERR("Error validating certificate\n");
-			SET_VERIFY_ERR_VARS(IERROR_CODE, IERROR_REASON);
+			SET_VERIFY_ERR_VARS(IERROR_CODE, IERROR_REASON_VALIDATE_CERTIFICATE);
 			goto error;
 		} else {  /* rc == -8 */
 			LM_INFO("Invalid certificate\n");
@@ -1835,19 +2144,19 @@ static int w_stir_verify(struct sip_msg *msg, str *cert_buf,
 		}
 	}
 
-	iat_ts = (time_t)parsed->iat->valuedouble;
-	if (iat_ts != date_ts && (now - iat_ts > verify_date_freshness))
+	if (date_hf && iat_ts != date_ts &&
+		(labs(now - iat_ts) > verify_date_freshness))
 		iat_ts = date_ts;
 
 	if ((rc = verify_signature(cert, parsed, iat_ts, orig_tn_p, dest_tn_p)) <= 0) {
 		if (rc < 0) {
 			LM_ERR("Error while verifying signature\n");
-			SET_VERIFY_ERR_VARS(IERROR_CODE, IERROR_REASON);
+			SET_VERIFY_ERR_VARS(IERROR_CODE, IERROR_REASON_VERIFY_SIGNATURE);
 			rc = -1;
 			goto error;
 		} else {
 			LM_INFO("Signature did not verify successfully\n");
-			SET_VERIFY_ERR_VARS(INVALID_IDENTITY_CODE, INVALID_IDENTITY_REASON);
+			SET_VERIFY_ERR_VARS(INVALID_IDENTITY_CODE, INVALID_IDENTITY_REASON " (bad signature)");
 			rc = -9;
 			goto error;
 		}
@@ -1868,21 +2177,10 @@ error:
 
 static int w_stir_check(struct sip_msg *msg)
 {
-	struct hdr_field *identity_hdr;
 	struct parsed_identity *parsed;
 	int rc;
 
-	if (parse_headers(msg, HDR_EOH_F, 0) < 0) {
-		LM_ERR("Failed to parse headers\n");
-		return -1;
-	}
-
-	if (!(identity_hdr = get_header_by_static_name(msg, "Identity"))) {
-		LM_INFO("No Identity header found\n");
-		return -2;
-	}
-
-	if ((rc = get_parsed_identity(identity_hdr, &parsed)) < 0) {
+	if ((rc = get_parsed_identity( msg, &parsed)) < 0) {
 		if (rc == -1) {
 			LM_ERR("Failed to parse identity header\n");
 			return -1;
@@ -1892,12 +2190,8 @@ static int w_stir_check(struct sip_msg *msg)
 		}
 	}
 
-	if (str_strcmp(&parsed->ppt_hdr_param, _str(PPORT_HDR_PPT_VAL))) {
-		LM_INFO("Unsupported 'ppt' extension\n");
-		return -4;
-	}
 	if (parsed->alg_hdr_param.s &&
-		str_strcmp(&parsed->alg_hdr_param, _str(PPORT_HDR_ALG_VAL))) {
+		str_strcmp(&parsed->alg_hdr_param, const_str(PPORT_HDR_ALG_VAL))) {
 		LM_INFO("Unsupported 'alg'\n");
 		return -4;
 	}
@@ -1937,28 +2231,28 @@ static int w_stir_check_cert(struct sip_msg *msg, str *cert_buf)
 	return 1;
 }
 
-int pv_parse_identity_name(pv_spec_p sp, str *in)
+int pv_parse_identity_name(pv_spec_p sp, const str *in)
 {
 	if (!in || !in->s || !in->len) {
 		LM_ERR("Bad subname for $identity\n");
 		return -1;
 	}
 
-	if (!str_strcmp(in, _str("header")))
+	if (!str_strcmp(in, const_str("header")))
 		sp->pvp.pvn.u.isname.name.n = PV_HEADER;
-	else if (!str_strcmp(in, _str(PPORT_HDR_X5U)))
+	else if (!str_strcmp(in, const_str(PPORT_HDR_X5U)))
 		sp->pvp.pvn.u.isname.name.n = PV_HEADER_X5U;
-	else if (!str_strcmp(in, _str("payload")))
+	else if (!str_strcmp(in, const_str("payload")))
 		sp->pvp.pvn.u.isname.name.n = PV_PAYLOAD;
-	else if (!str_strcmp(in, _str(PPORT_PAYLOAD_ATTEST)))
+	else if (!str_strcmp(in, const_str(PPORT_PAYLOAD_ATTEST)))
 		sp->pvp.pvn.u.isname.name.n = PV_PAYLOAD_ATTEST;
-	else if (!str_strcmp(in, _str(PPORT_PAYLOAD_DEST)))
+	else if (!str_strcmp(in, const_str(PPORT_PAYLOAD_DEST)))
 		sp->pvp.pvn.u.isname.name.n = PV_PAYLOAD_DEST;
-	else if (!str_strcmp(in, _str(PPORT_PAYLOAD_IAT)))
+	else if (!str_strcmp(in, const_str(PPORT_PAYLOAD_IAT)))
 		sp->pvp.pvn.u.isname.name.n = PV_PAYLOAD_IAT;
-	else if (!str_strcmp(in, _str(PPORT_PAYLOAD_ORIG)))
+	else if (!str_strcmp(in, const_str(PPORT_PAYLOAD_ORIG)))
 		sp->pvp.pvn.u.isname.name.n = PV_PAYLOAD_ORIG;
-	else if (!str_strcmp(in, _str(PPORT_PAYLOAD_ORIGID)))
+	else if (!str_strcmp(in, const_str(PPORT_PAYLOAD_ORIGID)))
 		sp->pvp.pvn.u.isname.name.n = PV_PAYLOAD_ORIGID;
 	else {
 		LM_ERR("Bad subname for $identity\n");
@@ -1970,21 +2264,10 @@ int pv_parse_identity_name(pv_spec_p sp, str *in)
 
 int pv_get_identity(struct sip_msg *msg, pv_param_t *param, pv_value_t *res)
 {
-	struct hdr_field *identity_hdr;
 	struct parsed_identity *parsed;
 	int rc;
 
-	if (parse_headers(msg, HDR_EOH_F, 0) < 0) {
-		LM_ERR("Failed to parse headers\n");
-		return pv_get_null(msg, param, res);
-	}
-
-	if (!(identity_hdr = get_header_by_static_name(msg, "Identity"))) {
-		LM_INFO("No Identity header found\n");
-		return pv_get_null(msg, param, res);
-	}
-
-	if ((rc = get_parsed_identity(identity_hdr, &parsed)) < 0) {
+	if ((rc = get_parsed_identity( msg, &parsed)) < 0) {
 		if (rc == -1)
 			LM_ERR("Failed to parse identity header\n");
 		else
@@ -2052,3 +2335,84 @@ int pv_get_identity(struct sip_msg *msg, pv_param_t *param, pv_value_t *res)
 
 	return 0;
 }
+
+
+/*
+ * MI functions
+ */
+
+/*! \brief Reload stir ca_list and ca_dir shaken */
+mi_response_t *mi_stir_shaken_ca_reload(const mi_params_t *params, struct mi_handler *async_hdl)
+{
+	int rc;
+
+	rc = init_cert_ca_reload();
+	LM_ERR("Engage stir shaken ca reload, result <%i>\n", rc);
+
+	switch(rc) {
+		case 0:
+			/* Reload OK */
+			return init_mi_result_ok();
+			break;
+		case -1:
+			/* X509 Store Error */
+			return init_mi_error(500, MI_SSTR("Error X509 store object."));
+			break;
+		case -2:
+			/* ca_list param */
+			return init_mi_error(500, MI_SSTR("Error ca_list param."));
+			break;
+		case -3:
+			/* ca_dir param */
+			return init_mi_error(500, MI_SSTR("Error ca_dir param."));
+			break;
+		case -4:
+			/* Cert Error */
+			return init_mi_error(500, MI_SSTR("Error failed to load trusted CAs."));
+			break;
+		case -5:
+			/* Path Error */
+			return init_mi_error(500, MI_SSTR("Error failed to load the system-wide CA certificates."));
+			break;
+		default:
+			/* Any other case */
+			return init_mi_error(500, MI_SSTR("Error"));
+	}
+
+}
+
+/*! \brief Reload stir crl_list and crl_dir shaken */
+mi_response_t *mi_stir_shaken_crl_reload(const mi_params_t *params, struct mi_handler *async_hdl)
+{
+	int rc;
+
+	rc = init_cert_crl_reload();
+	LM_ERR("Engage stir shaken crl reload, result <%i>\n", rc);
+
+	switch(rc) {
+		case 0:
+			/* Reload OK */
+			return init_mi_result_ok();
+			break;
+		case -1:
+			/* X509 Store Error */
+			return init_mi_error(500, MI_SSTR("Error X509 store object."));
+			break;
+		case -2:
+			/* crl_list param */
+			return init_mi_error(500, MI_SSTR("Error crl_list param."));
+			break;
+		case -3:
+			/* crl_dir param */
+			return init_mi_error(500, MI_SSTR("Error crl_dir param."));
+			break;
+		case -4:
+			/* Cert Error */
+			return init_mi_error(500, MI_SSTR("Error failed to load trusted CRL."));
+			break;
+		default:
+			/* Any other case */
+			return init_mi_error(500, MI_SSTR("Error"));
+	}
+
+}
